@@ -1,23 +1,23 @@
-
-/*  $Id: HBCIJobImpl.java,v 1.5 2011/06/06 10:30:31 willuhn Exp $
-
-    This file is part of HBCI4Java
-    Copyright (C) 2001-2008  Stefan Palme
-
-    HBCI4Java is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    HBCI4Java is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+/**********************************************************************
+ *
+ * This file is part of HBCI4Java.
+ * Copyright (c) 2001-2008 Stefan Palme
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ **********************************************************************/
 
 package org.kapott.hbci.GV;
 
@@ -36,6 +36,7 @@ import java.util.regex.Pattern;
 import org.kapott.hbci.GV_Result.HBCIJobResult;
 import org.kapott.hbci.GV_Result.HBCIJobResultImpl;
 import org.kapott.hbci.callback.HBCICallback;
+import org.kapott.hbci.dialog.KnownReturncode;
 import org.kapott.hbci.exceptions.HBCI_Exception;
 import org.kapott.hbci.exceptions.InvalidArgumentException;
 import org.kapott.hbci.exceptions.InvalidUserDataException;
@@ -88,6 +89,9 @@ public abstract class HBCIJobImpl
                                           über den logfilter-Mechanimus geschützt werden soll */
     
     private String externalId;
+    private int loopCount = 0;
+    private boolean haveTan = false;
+    private boolean skip = false;
     
     private HashSet<String> indexedConstraints;
     
@@ -206,7 +210,7 @@ public abstract class HBCIJobImpl
                     // willuhn 2011-06-06 Segment-Versionen ueberspringen, die groesser als die max. zulaessige sind
                     if (maxAllowedVersion > 0 && version > maxAllowedVersion)
                     {
-                      HBCIUtils.log("skipping segment version " + version + " for task " + jobnameLL + ", larger than allowed version " + maxAllowedVersion, HBCIUtils.LOG_INFO);
+                      HBCIUtils.log("skipping segment version " + version + " for task " + jobnameLL + ", larger than allowed version " + maxAllowedVersion, HBCIUtils.LOG_DEBUG);
                       continue;
                     }
                     // merken der größten jemals aufgetretenen versionsnummer
@@ -260,7 +264,7 @@ public abstract class HBCIJobImpl
       if (version.equals(this.segVersion))
         return;
       
-      HBCIUtils.log("changing segment version for task " + this.jobName + " explicit from " + this.segVersion + " to " + version,HBCIUtils.LOG_INFO);
+      HBCIUtils.log("changing segment version for task " + this.jobName + " explicitly from " + this.segVersion + " to " + version,HBCIUtils.LOG_DEBUG);
 
       // Der alte Name
       String oldName = this.name;
@@ -724,10 +728,48 @@ public abstract class HBCIJobImpl
         }
     }
     
-    public void setContinueOffset(int loop)
+    /**
+     * Setzt den Offset-Parameter mit dem aktuellen Loop-Count.
+     * Der Counter wird jedesmal in fillJobResult erhoeht - also mit jedem neuen Ergebnis.
+     */
+    public void applyOffset()
     {
-        String offset=getContinueOffset(loop);
-        setLowlevelParam(getName()+".offset",(offset!=null)?offset:"");
+        final String offset = this.getContinueOffset();
+        this.setLowlevelParam(this.getName() + ".offset",(offset != null) ? offset : "");
+    }
+    
+    /**
+     * Liefert true, wenn fuer den Auftrag ein HKTAN erzeugt wurde.
+     * @return true, wenn fuer den Auftrag ein HKTAN erzeugt wurde.
+     */
+    public boolean haveTan()
+    {
+        return this.haveTan;
+    }
+    
+    /**
+     * Vermerkt den Auftrag als "HKTAN erzeugt".
+     */
+    public void tanApplied()
+    {
+        this.haveTan = true;
+    }
+    
+    /**
+     * Markiert den Auftrag als zu ueberspringend.
+     */
+    public void skip()
+    {
+        this.skip = true;
+    }
+    
+    /**
+     * Prueft, ob der Auftrag uebersprungen werden soll.
+     * @return true, wenn der Auftrag uebersprungen werden soll.
+     */
+    public boolean skipped()
+    {
+        return this.skip;
     }
     
     protected void setLowlevelParam(String key,String value)
@@ -761,49 +803,61 @@ public abstract class HBCIJobImpl
         return this.segVersion;
     }
 
-    /* stellt fest, ob für diesen Task ein neues Auftragssegment generiert werden muss.
-       Das ist in zwei Fällen der Fall: der Task wurde noch nie ausgeführt; oder der Task
-       wurde bereits ausgeführt, hat aber eine "offset"-Meldung zurückgegeben */
-    public boolean needsContinue(int loop)
+    /**
+     * Liefert den ggf erneut auszufuehrenden Job.
+     * Die Default-Implementierung liefert "this", wenn die Bank ein 3040 zurueckgemeldet hat.
+     * Das kann aber auch ein anderer sein, als "this". Naemlich bei HKTAN in Prozess-Variante #2. Dort liefert es stattdessen den eigentlichen GV.
+     * @return den ggf erneut auszufuehrenden Job.
+     */
+    public HBCIJobImpl redo()
     {
-        boolean needs=false;
-
-        if (executed) {
-            HBCIRetVal retval=null;
-            int        num=jobResult.getRetNumber();
-
-            for (int i=0;i<num;i++) {
-                retval=jobResult.getRetVal(i);
-                
-                if (retval.code.equals("3040") && retval.params.length!=0 && (--loop)==0) {
-                    needs=true;
-                    break;
-                }
-            }
-        }
-        else needs=true;
-            
-        return needs;
-    }
-
-    /* gibt (sofern vorhanden) den offset-Wert des letzten HBCI-Rückgabecodes
-       zurück */
-    private String getContinueOffset(int loop)
-    {
-        String ret=null;
-        int    num=jobResult.getRetNumber();
+        if (!this.redoAllowed())
+            return null;
         
-        for (int i=0;i<num;i++) {
-            HBCIRetVal retval=jobResult.getRetVal(i);
-
-            if (retval.code.equals("3040") && retval.params.length!=0 && (--loop)==0) {
-                ret=retval.params[0];
-                break;
-            }
-        }
-
-        return ret;
+        return (this.getContinueOffset() != null) ? this : null;
     }
+    
+    /**
+     * Wir erlauben per Default erstmal kein Redo bei einem 3040-Code. Es sei denn, im Job ist explizit uebeschrieben.
+     * Siehe https://homebanking-hilfe.de/forum/topic.php?p=150614#real150614
+     * @return true, wenn redo erlaubt ist.
+     */
+    protected boolean redoAllowed()
+    {
+        return false;
+    }
+    
+    /**
+     * Gibt (sofern vorhanden) den Wiederaufsetzpunkt des letzten HBCI-Rückgabecodes 3040 zurück.
+     * @return der Offset-Wert oder NULL.
+     */
+    private String getContinueOffset()
+    {
+        HBCIRetVal ret = this.getW3040(this.loopCount);
+        return ret != null ? ret.params[0] : null;
+    }
+    
+    /**
+     * Liefert den Rueckgabecode 3040 (fuer "Weitere Daten folgen"), insofern vorhanden und mit einem Parameter versehen.
+     * @param loop die Nummer des Durchlaufs, beginnend bei 0.
+     * @return der Rueckgabewert, insofern vorhanden. Sonst NULL.
+     */
+    private HBCIRetVal getW3040(int loop)
+    {
+        final int num = jobResult.getRetNumber();
+
+        for (int i=0;i<num;i++)
+        {
+            HBCIRetVal retval = jobResult.getRetVal(i);
+            String[] p = retval.params;
+            if (KnownReturncode.W3040.is(retval.code) && p != null && p.length > 0 && p[0] != null && p[0].length() > 0 && (--loop) == 0)
+                return retval;
+        }
+        
+        return null;
+    }
+
+
 
     /* füllt das Objekt mit den Rückgabedaten. Dazu wird zuerst eine Liste aller
        Segmente erstellt, die Rückgabedaten für diesen Task enthalten. Anschließend
@@ -812,7 +866,10 @@ public abstract class HBCIJobImpl
     public void fillJobResult(HBCIMsgStatus status,int offset)
     {
         try {
-            executed=true;
+            this.executed = true;
+            this.haveTan = false;
+            this.skip = false;
+            this.loopCount++;
             Properties result=status.getData();
 
             // nachsehen, welche antwortsegmente ueberhaupt
@@ -1225,17 +1282,17 @@ public abstract class HBCIJobImpl
             if (o != null)
             {
                 String s = o.toString();
-                HBCIUtils.log("value of \"cannationalacc\" overwritten in passport, value: " + s,HBCIUtils.LOG_INFO);
+                HBCIUtils.log("value of \"cannationalacc\" overwritten in passport, value: " + s,HBCIUtils.LOG_DEBUG);
                 return s.equalsIgnoreCase("J");
             }
         }
         
-        HBCIUtils.log("searching for value of \"cannationalacc\" in HISPAS",HBCIUtils.LOG_INFO);
+        HBCIUtils.log("searching for value of \"cannationalacc\" in HISPAS",HBCIUtils.LOG_DEBUG);
         
         // Ansonsten suchen wir in HISPAS - aber nur, wenn wir die Daten schon haben
         if (handler.getSupportedLowlevelJobs().getProperty("SEPAInfo") == null)
         {
-            HBCIUtils.log("no HISPAS data found",HBCIUtils.LOG_INFO);
+            HBCIUtils.log("no HISPAS data found",HBCIUtils.LOG_DEBUG);
             return false; // Ne, noch nicht. Dann lassen wir das erstmal weg
         }
         
@@ -1243,7 +1300,7 @@ public abstract class HBCIJobImpl
         // SEPAInfo laden und darüber iterieren
         Properties props = handler.getLowlevelJobRestrictions("SEPAInfo");
         String value = props.getProperty("cannationalacc");
-        HBCIUtils.log("cannationalacc=" + value,HBCIUtils.LOG_INFO);
+        HBCIUtils.log("cannationalacc=" + value,HBCIUtils.LOG_DEBUG);
         return value != null && value.equalsIgnoreCase("J");
     }
     
